import click
from data import connect
import pandas as pd
from lxml import etree
from pathlib import Path
import os
import csv

def map(rating:str) -> int:
    mapping = {
        'right' : 0,
        'left-center' : 1,
        'center' : 2,
        'left' : 3,
        'allsides' : 4,
        'right-center' : 5
    }
    return mapping[rating]


@click.command(name="bias:load")
def load() -> None:
    DB = connect()
    DATA_DIR = Path(os.environ['DATA_MINING_DATA_DIR'])
    f = str(DATA_DIR / "bias_ratings.csv")

    DB.sql(f"""
        create table bias_ratings as 
        select 
            row_number() over(order by b.publisher) as id
            ,b.*
        from read_csv_auto('{f}') b
    """)
@click.command(name="bias:normalize")
def normalize() -> None:
    DB = connect()

    DB.sql("""
        with cte as (
            select 
                s.publisher
                ,count(1) as stories
            from stories s
            group by s.publisher
        )
        select
            s.publisher
            ,s.stories
            ,b.publisher
            ,b.bias
        from bias_ratings b
        join cte s
        on s.publisher = b.publisher
        order by
        stories desc
        limit 15
    """)

    DB.sql("""
        with cte as (
            select 
                s.publisher
                ,count(1) as stories
            from stories s
            group by s.publisher
        )
        select
            sum(stories)
            ,avg(agree / disagree)
        from bias_ratings b
        join cte s
        on s.publisher = b.publisher
    """)

    DB.sql("""
        with cte as (
            select 
                s.publisher
                ,count(1) as stories
            from stories s
            group by s.publisher
        )
        select
            sum(s.stories) filter(where b.publisher is not null) as matched
            ,sum(s.stories) filter(where b.publisher is null) as unmatched
            ,cast(sum(s.stories) filter(where b.publisher is not null) as numeric)
            / sum(s.stories) filter(where b.publisher is null) as precent_matched
        from bias_ratings b
        right join cte s
        on s.publisher = b.publisher
    """)

    DB.sql("""
        select
        *
        from bias_ratings
        where publisher ilike '%CNN%'
        """)

@click.command(name='bias:debug')
def debug() -> None:
    DB = connect()
    DATA_DIR = Path(os.environ['DATA_MINING_DATA_DIR'])
    f = str(DATA_DIR / "bias_ratings.csv")

    DB.sql("""
        with cte as (
            select 
                outlet 
                ,count(1) as stories
                from stories 
                group by outlet
        ) 
        ,total as (
            select
                sum(stories) as total
            from cte
        )
        select
            cte.outlet
            ,cte.stories
            ,bias.outlet
            ,bias.lean
            ,sum(100 * (cte.stories / cast(total.total as float))) over() as rep
            ,total.total
        from cte
        join bias 
        on jaro_winkler_similarity(bias.outlet, cte.outlet) > 0.9
        cross join total.total
    """)

    DB.sql("""
        select
            outlet
            ,count(1) as stories
        from stories
        group by outlet
        order by count(1) desc
        limit 50
    """)

    outlets

@click.command(name='bias:parse')
def parse() -> None:
    """parse the save html page of allslides.com bias ratings into a normalized csv file"""
    DB = connect()
    DATA_DIR = Path(os.environ['DATA_MINING_DATA_DIR'])
    bias_html = DATA_DIR / 'allsides.html'

    parser = etree.HTMLParser()
    tree = etree.parse(str(bias_html), parser)
    root = tree.getroot()
    rows = root.xpath('//table[contains(@class,"views-table")]/tbody/tr')

    ratings = []
    for row in rows:
        rating = dict()
        publisher = row.xpath('./td[contains(@class, "source-title")]/a')[0].text
        rating['publisher'] = publisher

        bias = row.xpath('./td[contains(@class, "views-field-field-bias-image")]/a')[0].get('href')
        bias = bias.split('/')[-1]
        rating['bias'] = bias

        agree = row.xpath('.//span[contains(@class, "agree")]')[0].text
        disagree = row.xpath('.//span[contains(@class, "disagree")]')[0].text

        rating['agree'] = int(agree)
        rating['disagree'] = int(disagree)
        ratings.append(rating)
    df = pd.DataFrame(ratings)
    df.to_csv(DATA_DIR / 'bias_ratings.csv', sep="|", index=False, quoting=csv.QUOTE_NONNUMERIC)
