from datetime import date, timedelta
import datetime
import requests
from pathlib import Path
import click
from tqdm import tqdm
from data import data_dir, connect
from lxml import etree
import pandas as pd

@click.command(name='scrape:load')
@click.option('--directory', type=Path, default=data_dir(), show_default=True)
@click.option('--database', type=Path, default=data_dir() / "stories.duckdb", show_default=True)
def load(directory, database):
    stories = directory / "stories.csv"
    related = directory / "related.csv"
    db = connect()

    db.sql(f"""
        CREATE OR REPLACE TABLE stories AS
        SELECT 
            * 
        FROM read_csv_auto('{stories}')
    """)

    db.sql(f"""
        CREATE OR REPLACE TABLE related_stories AS
        SELECT 
            * 
        FROM read_csv_auto('{related}')
    """)
    db.close()

@click.command(name='scrape:download')
@click.option('-o', 'output_dir', type=Path, default=data_dir() / "memeorandum", show_default=True)
def download(output_dir):
    """download every day from 01/10/2005 to today from memeorandum.com"""
    day = timedelta(days=1)
    cur = date(2005, 10, 1)
    end = date.today()
    dates = []
    while cur <= end:
        if not (output_dir / f"{cur.strftime('%y-%m-%d')}.html").exists():
            dates.append(cur)
        cur = cur + day
    date_iter = tqdm(dates, postfix="test")
    for i in date_iter:
        date_iter.set_postfix_str(f"{i}")
        save_as = output_dir / f"{i.strftime('%y-%m-%d')}.html"
        if save_as.exists():
            continue
        url = f"https://www.memeorandum.com/{i.strftime('%y%m%d')}/h2000"
        r = requests.get(url)
        with open(save_as, 'w') as f:
            f.write(r.text)


@click.command(name='scrape:parse')
@click.option('-d', '--directory', type=Path, default=data_dir() / "memeorandum", show_default=True)
@click.option('-o', '--output_dir', type=Path, default=data_dir(), show_default=True)
def parse(directory, output_dir):
    """parse the html files on disk into a structured csv format."""
    directory = data_dir() / "memeorandum"
    parser = etree.HTMLParser()
    pages = [f for f in directory.glob("*.html")]
    published = []
    others = []
    # page = pages[0]
    page_iter = tqdm(pages, postfix="starting")
    for page in page_iter:
        page_iter.set_postfix_str(f"{page}")
        date = datetime.datetime.strptime(page.stem, '%y-%m-%d')
        tree = etree.parse(str(page), parser)
        root = tree.getroot()
        if root is None:
            print(f"error opening {page}")
            continue
        items = root.xpath("//div[contains(@class, 'item')]")

        # item = items[0]
        for item in items:
            out = dict()
            out['published_at'] = date
            citation = item.xpath('./cite')
            if not citation:
                continue
            author = citation[0]
            if author.text: 
                author = ''.join(author.text.split('/')[:-1]).strip()
            else:
                author = ''
            out['author'] = author
            try:
                publisher_url = citation[0].getchildren()[0].get('href')
                publisher = citation[0].getchildren()[0].text
            except IndexError as e:
                print(f"error with citation url: {page}")
            out['publisher'] = publisher
            out['publisher_url'] = publisher_url

            title = item.xpath('.//strong/a')[0].text
            out['title'] = title

            url = item.xpath('.//strong/a')[0].get('href')
            out['url'] = url

            item_id = hash((page.stem, url))
            out['id'] = item_id

            old_id = hash((title, page.stem, publisher_url))
            out['old_id'] = old_id
            published.append(out)

            related = item.xpath(".//span[contains(@class, 'mls')]/a")
            # relation = related[0]
            for relation in related:
                another = dict()
                another['url'] = relation.get('href')
                another['publisher'] = relation.text
                another['parent_id'] = item_id
                others.append(another)
    df = pd.DataFrame(published)
    df.to_csv(output_dir / 'stories.csv', sep='|', index=False)
    df = pd.DataFrame(others)
    df.to_csv(output_dir / 'related.csv', sep='|', index=False)

@click.command(name='scrape:normalize')
def normalize():
    """fix database after load. remove duplicates. create publishers."""
    DB = connect()
    DB.sql("""
        DELETE FROM stories
        WHERE id IN (
            WITH cte AS (
                SELECT
                    url
                    ,id
                    ,ROW_NUMBER() OVER(PARTITION BY url) AS url_ctn
                    ,ROW_NUMBER() OVER(PARTITION BY title) AS title_ctn
                FROM stories
            )
            SELECT
                id
            FROM cte
            WHERE url_ctn > 1
            OR title_ctn > 1
        )
    """)
    DB.sql("""
        CREATE OR REPLACE TABLE publishers AS
        with cte as (
            SELECT
                s.publisher
                ,s.publisher_url
            FROM stories s
            GROUP BY 
                s.publisher
                ,s.publisher_url
        ), together AS (
            SELECT
                COALESCE(cte.publisher, r.publisher) AS publisher
                ,cte.publisher_url
            FROM cte
            FULL OUTER JOIN related_stories r
            ON cte.publisher = r.publisher
        )
        SELECT
            ROW_NUMBER() OVER() as id
            ,t.*
        FROM together t
        GROUP BY
            publisher
            ,publisher_url
    """)
